//----------------------------------------------------------------------------//
//  SettingViewController.m
//  SARURIS
//  Created by Takashi Kawasaki.
//  Creation date    2012/1/27
//  Last update date 2012/5/18
//  Copyright MaiWay Co., Ltd. All rights reserved.
//----------------------------------------------------------------------------//

//-------------------------------------------------//
// M-1 ヘッダーファイルをインポートしようブロック
//-------------------------------------------------//
#import "SettingViewController.h"
#import "CONST.h"

#pragma mark -
#pragma mark M-2 クラスとアクセサメソッドの実装をしようブロック
//-------------------------------------------------//
// M-2 クラスとアクセサメソッドの実装をしようブロック
//-------------------------------------------------//

@implementation SettingViewController
// （アクセサメソッドはここでは不使用）

#pragma mark -
#pragma mark M-3 まずはここに処理を書こうブロック
//-------------------------------------------------//
// M-3 まずはここに処理を書こうブロック
//-------------------------------------------------//

- (void)viewDidLoad
{
    [super viewDidLoad];

    // ビューの設定をまとめて行うメソッドを呼び出します。
    [self configureView];
}

#pragma mark -
#pragma mark M-4 ビューを好きなだけイジろうブロック
//-------------------------------------------------//
// M-4 ビューを好きなだけイジろうブロック
//-------------------------------------------------//

- (void)configureView
{
    // ユーザーデフォルトから保存データを読み出します。
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    float audioVolume            = [userDefaults floatForKey:@"audioVolume"];
    
    // iPhone/iPad共通設定その1
    // 戻るボタン（画面の遷移はstoryboardで設定）
    [backButton_ setTitle:@"" forState: UIControlStateNormal];
    [backButton_ addTarget:self action:@selector(playSound:) forControlEvents:UIControlEventTouchUpInside];
    
    // iPhone固有の設定
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
        // 背景イメージ
        bgImgView_          = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"bg_setting.png"]];  
        bgImgView_.frame    = CGRectMake(0.0, 0.0, 320.0, 480.0);
        
        // 戻るボタン
        [backButton_ setImage:[UIImage imageNamed:@"btn_settingback.png"] forState:UIControlStateNormal];
        backButton_.frame   = CGRectMake(0.0, 440.0, 320.0, 40.0);
        
        // テーブル
        settingTableView_ = [[UITableView alloc]initWithFrame:CGRectMake(0.0, 80.0, 320.0, 360.0) style:UITableViewStyleGrouped];
	// iPad固有の設定
    }else {
        // 背景イメージ）
        bgImgView_          = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"bg_setting_ipad.png"]];
        bgImgView_.frame    = CGRectMake(0.0, 0.0, 768.0, 1024.0);
        
        // 戻るボタン
        [backButton_ setImage:[UIImage imageNamed:@"btn_settingback_ipad.png"] forState:UIControlStateNormal];
        backButton_.frame   = CGRectMake(0.0, 944.0, 768.0, 80.0);
        
        // テーブル
        settingTableView_ = [[UITableView alloc]initWithFrame:CGRectMake(48.0, 168.0, 672.0, 776.0) style:UITableViewStyleGrouped];
	}
    
    // 背景をビューに加えます。（storyboard上の各ボタンの下に設置）
    [self.view insertSubview:bgImgView_ atIndex:0];
    
    // iPhone/iPad共通設定その2
    // テーブルを設定
    settingTableView_.backgroundView    = nil;
    settingTableView_.backgroundColor   = [UIColor clearColor];
    settingTableView_.separatorColor    = [UIColor darkGrayColor];
    settingTableView_.separatorStyle    = UITableViewCellSeparatorStyleSingleLine;
    
    // デリゲートを設定
    [settingTableView_ setDataSource:self];
    [settingTableView_ setDelegate:self];
    
    // テーブルのリロード
    [settingTableView_ reloadData];
    
    // テーブルをビューに加えます。
    [self.view addSubview:settingTableView_];
    
    // ボタン押下時の効果音を準備する
	NSString *soundPath = [[NSBundle mainBundle] pathForResource:@"se_cancel" ofType:@"wav"];
	NSURL *soundUrl     = [NSURL fileURLWithPath:soundPath];
    AudioServicesCreateSystemSoundID((__bridge CFURLRef)soundUrl, &gameSound_);
    
    // BGMの再生を準備する（繰り返し設定）
    NSString *audioPath         = [[NSBundle mainBundle] pathForResource:@"bgm_setting" ofType:@"mp3"];  
    NSURL *audioUrl             = [NSURL fileURLWithPath:audioPath];
    gameAudio_                  = [[AVAudioPlayer alloc] initWithContentsOfURL:audioUrl error:nil]; 
    gameAudio_.currentTime      = 0.0;
    gameAudio_.volume           = audioVolume;
    gameAudio_.numberOfLoops    = -1;  
    [gameAudio_ prepareToPlay];
}

#pragma mark -
#pragma mark M-5 オリジナルメソッドを実装しようブロック
//-------------------------------------------------//
// M-5 オリジナルメソッドを実装しようブロック
//-------------------------------------------------//

// BGM再生ボタンが押された時のアクションを設定します。
- (void)playSound:(id)sender
{
    // 音を鳴らす
	AudioServicesPlaySystemSound(gameSound_);
}

// 難易度が変更された時のアクションを設定します。
- (void)changeDifficulty:(UISegmentedControl *)seg
{
    // ユーザーデフォルトの保存
    NSUserDefaults *userDefaults    = [NSUserDefaults standardUserDefaults];
    [userDefaults setInteger:seg.selectedSegmentIndex forKey:@"difficulty"];    
}

// ボタン透明度が変更された時のアクションを設定します。
- (void)changeAlpha:(UISlider *)slider
{
    // ユーザーデフォルトの保存
    NSUserDefaults *userDefaults    = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:slider.value forKey:@"buttonAlpha"];
}

// ボリュームが変更された時のアクションを設定します。
- (void)changeVolume:(UISlider *)slider
{
    // ユーザーデフォルトの保存
    NSUserDefaults *userDefaults    = [NSUserDefaults standardUserDefaults];
    [userDefaults setFloat:slider.value forKey:@"audioVolume"];
    gameAudio_.volume = slider.value;
}

// ランキングをリセットしていいか確認します。
- (void)resetAlert
{
    // アラートタイプを設定
    switch (alertType_) {
        case kImageAlert:
        {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Confirm", @"")
                                                            message:NSLocalizedString(@"Do you want to reset Background Photo?", @"")
                                                           delegate:self
                                                  cancelButtonTitle:NSLocalizedString(@"No", @"")
                                                  otherButtonTitles:NSLocalizedString(@"Yes", @""), nil];
            [alert show];
        }
            break;
        case kMusicAlert:
        {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Confirm", @"")
                                                            message:NSLocalizedString(@"Do you want to reset Background Music?", @"")
                                                           delegate:self
                                                  cancelButtonTitle:NSLocalizedString(@"No", @"")
                                                  otherButtonTitles:NSLocalizedString(@"Yes", @""), nil];
            [alert show];
        }
            break;
        case kRankingAlert:
        {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Confirm", @"")
                                                            message:NSLocalizedString(@"Do you want to reset Ranking?", @"")
                                                           delegate:self
                                                  cancelButtonTitle:NSLocalizedString(@"No", @"")
                                                  otherButtonTitles:NSLocalizedString(@"Yes", @""), nil];
            [alert show];
        }
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark M-6 デリゲートメソッドをイジろうブロック
//-------------------------------------------------//
// M-6 デリゲートメソッドをイジろうブロック
//-------------------------------------------------//

// アラートの各ボタンが押された時の処理です。
- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{    
    switch (buttonIndex) {
        // キャンセル時
		case 0: {
            // ハイライト時の背景色を戻す
            [settingTableView_ deselectRowAtIndexPath:[settingTableView_ indexPathForSelectedRow] animated:YES];
			break;
		}
        // 決定時
		case 1: {
            // ユーザーデフォルトの保存
            NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
            
            switch (alertType_) {
                case kRankingAlert:
                {
                    NSArray *basicArray = [[NSArray alloc] initWithObjects:@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0",@"0", nil];
                    [userDefaults setObject:basicArray forKey:@"rankingData"];
                }
                    break;
                case kMusicAlert:
                {
                    NSString *backgroundMusicTitle = @"";
                    [userDefaults setObject:backgroundMusicTitle forKey:@"backgroundMusicTitle"];
                    
                    // テーブルのリロード
                    [settingTableView_ reloadData];
                }
                    break;
                case kImageAlert:
                {
                    NSData *backgroundImageData =  [[NSData alloc] initWithData:UIImagePNGRepresentation([UIImage imageNamed:@"bg_game.png"])];
                    [userDefaults setObject:backgroundImageData forKey:@"backgroundImageData"];
                    
                    // テーブルのリロード
                    [settingTableView_ reloadData];
                }
                    break;
                default:
                    break;
            }
            
            // ハイライト時の背景色を戻す
            [settingTableView_ deselectRowAtIndexPath:[settingTableView_ indexPathForSelectedRow] animated:YES];
			break;
		}
	}
}

// 枠外を触ったときに、ポップオーバービューを閉じます。
- (BOOL)popoverControllerShouldDismissPopover:(UIPopoverController *)popoverController
{
    return YES;
}

// 閉じた後、選択色を消します。
- (void)popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
    // テーブルの選択色をクリア
    [settingTableView_ deselectRowAtIndexPath:[settingTableView_ indexPathForSelectedRow] animated:YES];
    return;
}

// カメラとアルバムで取得したイメージを編集します。
- (void)imagePickerController:(UIImagePickerController*)picker didFinishPickingMediaWithInfo:(NSDictionary*)info
{
    // イメージの入れ物
    UIImage *originalImage;
    CGSize size;
    
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){   
        // イメージピッカーを隠す
        [self dismissModalViewControllerAnimated:YES];
        
        // オリジナル画像を取得する
        originalImage = [info objectForKey:UIImagePickerControllerOriginalImage];
        
        // グラフィックスコンテキストを作る
        size = CGSizeMake(320.0, 480.0);
    }else{
        // UIPopoverControllerを閉じる
        [popoverController_ dismissPopoverAnimated:YES];
        
        // オリジナル画像を取得する
        originalImage = [info objectForKey:UIImagePickerControllerOriginalImage];
        
        // グラフィックスコンテキストを作る
        size = CGSizeMake(768.0, 1024.0);
    }
    
    // グラフィックスコンテキストを作る
    UIGraphicsBeginImageContext(size);
    
	// 画像を縮小して描画する
	CGRect rect = CGRectMake(0.0, 0.0, 0.0, 0.0);
	rect.size = size;
	[originalImage drawInRect:rect];
	
	// 描画した画像を取得する
	UIImage *shrinkedImage;
	shrinkedImage = UIGraphicsGetImageFromCurrentImageContext();
	UIGraphicsEndImageContext();
	
    // ユーザーデフォルトの保存
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    NSData *backgroundImageData =  [[NSData alloc] initWithData:UIImagePNGRepresentation(shrinkedImage)];
    [userDefaults setObject:backgroundImageData forKey:@"backgroundImageData"];
    
    // テーブルのリロード
    [settingTableView_ reloadData];
}

// iPodライブラリから音楽を取得します。
- (void)mediaPicker: (MPMediaPickerController *) mediaPicker didPickMediaItems: (MPMediaItemCollection *) mediaItemCollection
{
    // iPodライブラリ画面のクローズ
    [self dismissModalViewControllerAnimated: YES];
    
    // 音楽タイトルの取得
    userMediaItemCollection_ = mediaItemCollection;
    MPMediaItem *representativeItem = [userMediaItemCollection_ representativeItem];
    NSString *title = [representativeItem valueForProperty: MPMediaItemPropertyTitle];
    
    // ユーザーデフォルトの保存
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    [userDefaults setObject:title forKey:@"backgroundMusicTitle"];
    
    // テーブルのリロード
    [settingTableView_ reloadData];
}

// iPodライブラリをキャンセルして画面を戻します。
- (void)mediaPickerDidCancel:(MPMediaPickerController *)mediaPicker
{
    // iPodライブラリ画面のクローズ
	[self dismissModalViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark M-6.0 テーブルのあれこれをイジろうブロック
//------------------------------------------//
// M-6.0 テーブルのあれこれをイジろうブロック
//------------------------------------------//

// テーブルのセクション数を設定します。
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 5;
}

// テーブルの行数を設定します。
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    switch (section) {
        case 0:
            return 1;
            break;
        case 1:
            return 2;
            break;
        case 2:
            return 2;
            break;
        case 3:
            return 3;
            break;
        case 4:
            return 2;
            break;
        default:
            break;
    }
	return 1;
}

// テーブル列の高さを設定します。
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
	return 40;
}

// 各セクションのヘッダータイトルを設定します。
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    switch (section) {
        case 0:
            return @"Difficulty Level";
            break;
        case 1:
            return @"User Settings";
            break;
        case 2:
            return @"Custom Settings";
            break;
        case 3:
            return @"Reset Settings";
            break;
        case 4:
            return @"Information";
            break;
        default:
            break;
    }
	return @"";
}

// 各セクションのヘッダータイトルをカスタマイズします。
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
	UIView *tempView = [[UIView alloc] init];
    tempView.backgroundColor = [UIColor clearColor];
    
	UILabel *tempLabel = [[UILabel alloc] initWithFrame:CGRectMake(20, 0, 300, 30)];
    tempLabel.backgroundColor = [UIColor clearColor];
	tempLabel.textColor = [UIColor whiteColor];
    
    switch (section) {
        case 0:
            tempLabel.text = NSLocalizedString(@"Difficulty Level", @"");
            break;
        case 1:
            tempLabel.text = NSLocalizedString(@"User Settings", @"");
            break;
        case 2:
            tempLabel.text = NSLocalizedString(@"Custom Settings", @"");
            break;
        case 3:
            tempLabel.text = NSLocalizedString(@"Reset Settings", @"");
            break;
        case 4:
            tempLabel.text = NSLocalizedString(@"Information", @"");
            break;
        default:
            break;
    }
    
    [tempView addSubview:tempLabel];
    return tempView;
}

// テーブルの見た目を設定します。
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{    
    // ユーザーデフォルトの読み出し
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    float audioVolume            = [userDefaults floatForKey:@"audioVolume"];
    float buttonAlpha            = [userDefaults floatForKey:@"buttonAlpha"];
    NSInteger difficulty         = [userDefaults integerForKey:@"difficulty"];
    
    // 現在のセクション／行番号を取得
    NSInteger section = [indexPath section];
	NSInteger row     = [indexPath row];
    
    // セルを種類別に固有化する（各種類ごとにセルを分類）
    static const id CellIdentifiers[5] = {@"seg-cell", @"slider-cell1", @"text-cell1",@"btn-cell",@"text-cell2"};
    NSString* CellIdentifier = CellIdentifiers[indexPath.section];
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    
    // セルの新規作成時の設定
    if (cell == nil) {        
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier];
        
        // 難易度セグメント
        if(section == 0){
            NSString *easy   = NSLocalizedString(@"Easy", @"");
            NSString *medium = NSLocalizedString(@"Medium", @"");
            NSString *hard   = NSLocalizedString(@"Hard", @"");
            NSArray *items   = [NSArray arrayWithObjects:easy, medium, hard, nil];
            
            // 上記の表題で3つのボタンを作る
            UISegmentedControl *difficultySeg = [[UISegmentedControl alloc] initWithItems:items];
            // iPhoneの場合
            if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
                difficultySeg.frame = CGRectMake(0.0, 0.0, 300.0, 40.0);
            // iPadの場合
            }else{
                difficultySeg.frame = CGRectMake(0.0, 0.0, 590.0, 40.0);
                
            }                
            difficultySeg.selectedSegmentIndex = difficulty;
            // メソッドを設定する
            [difficultySeg addTarget:self action:@selector(changeDifficulty:) forControlEvents:UIControlEventValueChanged];
            // セルに追加する
            [cell.contentView addSubview:difficultySeg];
        }else if(section == 1){
            // ボリュームスライダー
            if (row == 0) {
                UISlider *volumeSlider = [[UISlider alloc] init];
                // iPhoneの場合
                if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
                    volumeSlider.frame = CGRectMake(170.0, 0.0, 125.0, 40.0);
                // iPadの場合
                }else{
                    volumeSlider.frame = CGRectMake(330.0, 0.0, 250.0, 40.0);
                }
                // 最小値を設定する
                volumeSlider.minimumValue = 0.1;
                // 最大値を設定する
                volumeSlider.maximumValue = 1.0;
                // スライダーの初期値を設定する
                volumeSlider.value = audioVolume;
                // スライド中にイベントが通知されるように設定する
                volumeSlider.continuous = YES;
                // タグを設定する
                volumeSlider.tag = kSliderTag2;
                // メソッドを設定する
                [volumeSlider addTarget:self action:@selector(changeVolume:) forControlEvents:UIControlEventValueChanged];
                // セルに追加する
                [cell.contentView addSubview: volumeSlider];
            // 透明度スライダー
            }else if(row == 1){
                UISlider *alphaSlider = [[UISlider alloc] init];
                // iPhoneの場合
                if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
                    alphaSlider.frame = CGRectMake(170.0, 0.0, 125.0, 40.0);
                // iPadの場合
                }else{
                    alphaSlider.frame = CGRectMake(330.0, 0.0, 250.0, 40.0);
                }
                // 最小値を設定する
                alphaSlider.minimumValue = 0.1;
                // 最大値を設定する
                alphaSlider.maximumValue = 1.0;
                // スライダーの初期値を設定する
                alphaSlider.value = buttonAlpha;
                // スライド中にイベントが通知されるように設定する
                alphaSlider.continuous = YES;
                // タグを設定する
                alphaSlider.tag = kSliderTag3;
                // メソッドを設定する
                [alphaSlider addTarget:self action:@selector(changeAlpha:) forControlEvents:UIControlEventValueChanged];
                // セルに追加する
                [cell.contentView addSubview: alphaSlider];
            }
            // セルが選択されたときの選択色をなしに
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
        }else if(section == 2){
            // カスタマイズセクション
            if (row == 0) {
                // ユーザーデフォルトから保存データを読み出し
                NSUserDefaults *userDefaults   = [NSUserDefaults standardUserDefaults];
                UIImageView *customImage= [[UIImageView alloc] initWithImage:[UIImage imageWithData:[userDefaults dataForKey:@"backgroundImageData"]]];
                
                // iPhoneの場合
                if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
                    customImage.frame = CGRectMake(255.0, 5.0, 30.0, 30.0);
                // iPadの場合
                }else{
                    customImage.frame = CGRectMake(540.0, 5.0, 30.0, 30.0);
                }
                // タグを設定する
                customImage.tag = kImageTag1;
                // セルに追加する
                [cell.contentView addSubview:customImage];
            }
        }else if(section == 3){
            // リセットセクション
            UILabel *customLabel = [[UILabel alloc] init];
            
            // iPhoneの場合
            if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){
                customLabel.frame = CGRectMake(0.0, 0.0, 300.0, 40.0);
            // iPadの場合
            }else{
                customLabel.frame = CGRectMake(0.0, 0.0, 590.0, 40.0);
            }
            customLabel.textAlignment = UITextAlignmentCenter;
            customLabel.textColor = [UIColor whiteColor];
            customLabel.backgroundColor = [UIColor clearColor];
            [cell.contentView addSubview:customLabel];
            // タグを設定する
            if (row == 0) {
                customLabel.tag = kLabelTag1;
            }else if(row == 1){
                customLabel.tag = kLabelTag2;
            }else if(row == 2){
                customLabel.tag = kLabelTag3;
            }
        }
    }
    
    // セルの毎回表示時の設定
    // セルの背景の設定
    cell.backgroundColor = [UIColor clearColor];
	
	// セルの文字の設定
    cell.textLabel.textColor = [UIColor whiteColor];
	cell.textLabel.font = [UIFont boldSystemFontOfSize:18];
    cell.textLabel.textAlignment = UITextAlignmentLeft;
    cell.textLabel.adjustsFontSizeToFitWidth =  YES;
    cell.textLabel.minimumFontSize = 8;
    
    // セルの文字の設定（補足文字）
    cell.detailTextLabel.textColor = [UIColor lightGrayColor];
    cell.detailTextLabel.font = [UIFont boldSystemFontOfSize:14];
    cell.detailTextLabel.textAlignment = UITextAlignmentRight;
    cell.detailTextLabel.adjustsFontSizeToFitWidth =  YES;    
    cell.detailTextLabel.minimumFontSize = 8;
    
    // 難易度セグメント
    if(section == 0){
        // セルのタイトル
        cell.textLabel.text = @"";
        cell.detailTextLabel.text = @"";
        // セルが選択されたときの選択色をなしに
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }else if(section == 1){
        // ボリュームスライダー
        if (row == 0) {
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Volume", @"");
            cell.detailTextLabel.text = @"";
        // 透明度スライダー
        }else if(row == 1){
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Transparency", @"");
            cell.detailTextLabel.text = @"";
        }
        // セルが選択されたときの選択色をなしに
        cell.selectionStyle = UITableViewCellSelectionStyleNone;     
    // カスタマイズセクション
    }else if(section == 2){
        if (row == 0) {
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Photo", @"");
            cell.detailTextLabel.text = @"";
            
            // ユーザーデフォルトの読み出し
            UIImageView *imageVIew = (UIImageView *)[cell.contentView viewWithTag:kImageTag1];
            imageVIew.image = [UIImage imageWithData:[userDefaults dataForKey:@"backgroundImageData"]];
        }else if(row == 1){
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Music", @"");
            
            // ユーザーデフォルトの読み出し
            NSString *backgroundMusicTitle = [userDefaults stringForKey:@"backgroundMusicTitle"];
            if ([backgroundMusicTitle length] == 0) {
                cell.detailTextLabel.text = NSLocalizedString(@"Default", @"");
            }else{
                cell.detailTextLabel.text = backgroundMusicTitle;
            }
        }
        // セルが選択されたときの選択色をなしに
        cell.selectionStyle = UITableViewCellSelectionStyleGray;
    // リセットセクション
    }else if(section == 3){
        if (row == 0) {
            UILabel *label = (UILabel *)[cell.contentView viewWithTag:kLabelTag1];
            label.text = NSLocalizedString(@"Photo", @"");
        }else if(row == 1){
            UILabel *label = (UILabel *)[cell.contentView viewWithTag:kLabelTag2];
            label.text = NSLocalizedString(@"Music", @"");
        }else if(row == 2){
            UILabel *label = (UILabel *)[cell.contentView viewWithTag:kLabelTag3];
            label.text = NSLocalizedString(@"Ranking", @"");
        }
        // セルが選択されたときの選択色をグレーに
        cell.selectionStyle = UITableViewCellSelectionStyleGray;
    // 情報セクション
    }else if(section == 4){
        if (row == 0) {
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Version", @"");
            cell.detailTextLabel.text = kVersion;
        }else if(row == 1){
            // セルのタイトル
            cell.textLabel.text = NSLocalizedString(@"Copyright", @"");
            cell.detailTextLabel.text = kCopyRight;
        }
        // セルが選択されたときの選択色をなしに
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
	return cell;
}

// 行の選択開始時の処理を設定します。
- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    return indexPath;
}

// 行の選択後の処理を設定します。
- (void)tableView:(UITableView * )tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // 現在のセクション／行番号を取得
    NSInteger section = [indexPath section];
	NSInteger row     = [indexPath row];
    
    if (section == 2) {
        // 背景写真のカスタマイズ
        if (row == 0) {
            // 設定されたソースタイプが使用可能かどうかチェックする
            if (![UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypePhotoLibrary]) {  
                return;
            }
            
            // イメージピッカーを生成する
            imagePickerController_ = [[UIImagePickerController alloc] init];
            imagePickerController_.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
            imagePickerController_.delegate = self;
            
            // iPhoneの場合
            if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPhone){             
                // イメージピッカーをモーダル画面として表示する
                [self presentModalViewController:imagePickerController_ animated:YES];
            // iPadではフルスクリーンのモーダル画面は非推奨なので、ポップオーバービューを使う
            }else{
                //UIImagePickerControllerの入ったUIPopoverControllerを作成する方法
                popoverController_ = [[UIPopoverController alloc] initWithContentViewController:imagePickerController_];
                popoverController_.delegate = self;
                
                //UIPopoverControllerを表示する方法
                if(!popoverController_.popoverVisible) {
                    [popoverController_ presentPopoverFromRect:CGRectMake(0.0, 910.0, 768.0, 40.0) inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES]; 
                }
            }
        // BGMのカスタマイズ
        }else if (row == 1) {
            mediaPickerController_ = [[MPMediaPickerController alloc] init];
            mediaPickerController_.allowsPickingMultipleItems = NO;
            mediaPickerController_.delegate = self;
            // メディアピッカーをモーダル画面として表示する
            [self presentModalViewController:mediaPickerController_ animated: YES];
        }
    // リセットセクション
    }else if(section == 3){
        if(row == 0){
            alertType_ = kImageAlert;
        }else if(row == 1){
            alertType_ = kMusicAlert;
        }else if(row == 2){
            alertType_ = kRankingAlert;
        }
        [self resetAlert];
    }
}

#pragma mark -
#pragma mark M-6.1 ビューのあれこれをイジろうブロック
//------------------------------------------//
// M-6.1 ビューのあれこれをイジろうブロック
//------------------------------------------//

// ビューが表示される「前」にこの処理が呼ばれます。
- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    // BGM再生
    [gameAudio_ play];
    
    // テーブルの選択色をクリア
    [settingTableView_ deselectRowAtIndexPath:[settingTableView_ indexPathForSelectedRow] animated:YES];
}

// ビューが表示された「後」にこの処理が呼ばれます。
- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

// ビューが非表示にされる「前」にこの処理が呼び出されます。
- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
    
    // 音楽を止めて初期化
    if ([gameAudio_ isPlaying]) {
        [gameAudio_ stop];
        gameAudio_.currentTime = 0.0;
    }
}

// ビューが非表示にされた「後」にこの処理が呼び出されます。
- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

#pragma mark -
#pragma mark M-6.2 ローテーションをイジろうブロック
//------------------------------------------//
// M-6.2 ローテーションをイジろうブロック
//------------------------------------------//

// iPhone/iPadが倒された時にローテーションをサポートするかどうかを設定します。
// YESが返るようにするとその方向に対してローテーションが作動するようになります。
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // iPhoneの場合
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        return (interfaceOrientation == UIInterfaceOrientationPortrait                  //正しい向き
                //||interfaceOrientation == UIInterfaceOrientationLandscapeLeft         //右に倒した場合
                //||interfaceOrientation == UIInterfaceOrientationLandscapeRight		//左に倒した場合
                //||interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown	//逆さまにした場合
                );
    // iPadの場合
    } else {
        return (interfaceOrientation == UIInterfaceOrientationPortrait                  //正しい向き
                //||interfaceOrientation == UIInterfaceOrientationLandscapeLeft         //右に倒した場合
                //||interfaceOrientation == UIInterfaceOrientationLandscapeRight		//左に倒した場合
                //||interfaceOrientation == UIInterfaceOrientationPortraitUpsideDown	//逆さまにした場合
                );
    }
}

#pragma mark -
#pragma mark M-6.3 メモリー管理をイジろうブロック
//------------------------------------------//
// M-6.3 メモリー管理をイジろうブロック
//------------------------------------------//

// View Controllerに関連付けられているカスタムデータ構造のうち、
// 重要でないものを解放するには、このメソッドを使用します。なんてアップルも言っています。
- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

// インスタンス化したオブジェクトのメモリを解放します。
- (void)viewDidUnload
{
    [super viewDidUnload];
}

//-------------------------------------------------//
// M-7 メソッドファイルおつかれさまでしたブロック
//-------------------------------------------------//
@end
